#version 430 core

in vec2 TexCoord0;
in vec4 orig_position;
flat in float TTL0;

uniform sampler2D gSampler; 
uniform float minTTL;
uniform float maxTTL;

// motion vector calculation requirement
uniform mat4 prevModelMatrix;
uniform mat4 prevVPMatrix;
uniform mat4 vpMatrix;
uniform mat4 modelMatrix;

out vec4 FragColor;

layout (location = 0) out vec4 gFragColor;
layout (location = 1) out vec2 gMotionVec;

void main() {
    vec4 vTexColor = texture2D(gSampler, TexCoord0); 
    // vec4 vTexColor = vec4(.5, .5, 0, 1);
    float alpha = TTL0 / (maxTTL - minTTL);
    if(vTexColor.x == 0 && vTexColor.y == 0 && vTexColor.z == 0){
        alpha = 0;
    }

    FragColor = vec4(vTexColor.xyz, alpha);

    // calculate motion vector for each pixel
    vec4 previous_position = prevVPMatrix * prevModelMatrix * orig_position;
    previous_position /= previous_position.w;

    vec4 current_position = vpMatrix * modelMatrix * orig_position;
    current_position /= current_position.w;

    gFragColor = vec4(vTexColor.xyz, alpha);
    gMotionVec = (previous_position.xy - current_position.xy)/2.f;
}
