#version 430
layout(local_size_x = 16, local_size_y = 16, local_size_z = 1) in;

layout(std430, binding=0) buffer pos1
{
    vec4 pos_in[];      // (x, y, z, ttl)
};
layout(std430, binding=2) buffer pos2
{
    vec4 pos_out[];
};
layout(std430, binding=1) buffer vel1
{
    vec4 vel_in[];      // (x, y, z, null)
};
layout(std430, binding=3) buffer vel2
{
    vec4 vel_out[];
};

layout (binding = 4, offset = 0) uniform atomic_uint _count;

uniform vec4 spawnPos;
uniform uint lastCount;
uniform uint maxCount;
uniform float deltaT;
uniform vec3 gravity;

void main()
{
    uint idx = gl_GlobalInvocationID.x + gl_GlobalInvocationID.y * gl_NumWorkGroups.x * gl_WorkGroupSize.x;
    
    if(idx >= lastCount) return;

    float ttl = pos_in[idx].w - deltaT;
    vec3 pos = vec3(pos_in[idx].x, pos_in[idx].y, pos_in[idx].z);
    vec4 vel = vel_in[idx];
    // vel = (vel + vec4(gravity, 0)) * deltaT;
    vel = vel * deltaT;

    pos += vel.xyz;

    if(ttl <= 0){
        atomicCounterIncrement(_count);
        ttl = spawnPos.w;
        pos = vec3(spawnPos.xyz);
    }

    uint nr = atomicCounterIncrement(_count);
    if(nr >= maxCount)
    {
        atomicCounterDecrement(_count);
        return;
    }
    pos_out[nr] = vec4(pos, ttl);
    vel_out[nr] = vel;
}